# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals

import re

class AtWikiStripper(object):
  # Comment: `// comment`
  COMMENT = re.compile(r'^//')

  # Inline annotation: `&color(#999999){text}`, `&nicovideo(url)`
  INLINE_ANN = re.compile(r'&[a-z_]+\(([^()]*?)\)({([^{}]+?)})?'), 3

  # Inline links: `[[page]]`, `[[alias>URL]]`
  INLINE_LINK = re.compile(r'\[\[(.+?)((>|>>)(.+?))?\]\]'), 1

  # Inline italic: `'''text'''`
  INLINE_ITALIC = re.compile(r'\'\'\'(.+?)\'\'\''), 1

  # Inline bold: `''text''`
  INLINE_BOLD = re.compile(r'\'\'(.+?)\'\''), 1

  # Inline del: `%%text%%`
  INLINE_DEL = re.compile(r'%%(.+?)%%'), 1

  # Line annotation: `#right(){text}`, `#comment()`, `#region`
  LINE_ANN = re.compile(r'^#[a-z_]+(\(([^()]*?)\)({([^{}]+?)})?)?\s*$'), 4

  # Line horizontal line: `----`
  LINE_HR = re.compile(r'^----\s*()$'), 1

  # Line item list and heading: `+foo`, `-foo`, `*foo`
  LINE_ITEMLIST = re.compile(r'^(\*+|\++|-+)(.+)$'), 2

  # Line quote: `>text`
  LINE_QUOTE = re.compile(r'^>+(.+)$'), 1

  # Line formatted: ` text`
  LINE_PRE = re.compile(r'^ (.+)$'), 1

  # Block annotation: `#exk(){{{` ... `}}}`
  BLOCK_BEGIN_ANN = re.compile(r'^#[a-z_]+\(([^{}()]*?)\)({+)\s*$')
  BLOCK_END_ANN = re.compile(r'^(}+)\s*$')

  def __init__(self, source):
    self._source = source

  def _inline_strip(self, line, pattern, group):
    while True:
      prev = line
      # Note: prior to Python 3.5, use of backreference of nonmatching group
      # in replacement string raises exception.
      line = pattern.sub(lambda m: m.group(group), line)
      if prev == line: return line

  def _line_process(self, buf, line, pattern, group):
    prev = line
    line = pattern.sub(lambda m: m.group(group), line)
    if prev == line: return False
    buf.append(line)
    return True

  def text(self):
    ret = []
    lines = self._source.splitlines()
    block_level = 0
    for line in lines:
      if self.COMMENT.match(line): continue
      line = self._inline_strip(line, *self.INLINE_ANN)
      line = self._inline_strip(line, *self.INLINE_LINK)
      line = self._inline_strip(line, *self.INLINE_ITALIC)
      line = self._inline_strip(line, *self.INLINE_BOLD)
      line = self._inline_strip(line, *self.INLINE_DEL)
      if self._line_process(ret, line, *self.LINE_ANN): continue
      if self._line_process(ret, line, *self.LINE_HR): continue
      if self._line_process(ret, line, *self.LINE_ITEMLIST): continue
      if self._line_process(ret, line, *self.LINE_QUOTE): continue
      if self._line_process(ret, line, *self.LINE_PRE): continue
      if block_level == 0:
        m = self.BLOCK_BEGIN_ANN.match(line)
        if m:
          block_level = len(m.group(2))
          continue
      else:
        m = self.BLOCK_END_ANN.match(line)
        if m and len(m.group(1)) == block_level:
          block_level = 0
          continue
      ret.append(line)
    return '\n'.join(ret)



# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals

try:
  # Python 3
  import urllib.parse as urlparse
  from urllib.parse import quote as urlquote
except:
  # Python 2
  import urlparse
  def urlquote(s, safe=b'/'):
    import urllib
    if isinstance(s, unicode):
      s = s.encode('utf-8')
    return urllib.quote(s, safe)

import os.path
import warnings


class AtWikiURI(object):
  def __init__(self, base):
    self._base = base

  def __str__(self):
    return self._base

  ########################################
  # URI Transformations
  ########################################

  def get_page_id_from_uri(self, uri):
    p = urlparse.urlparse(self.get_absolute_uri(uri))

    # backupx
    backupx = p.path.split('/')[-3:]
    if len(backupx) == 3 and backupx[0] == 'backupx' and backupx[2] == 'list.html':
      return int(backupx[1])

    # special pages
    pageid = urlparse.parse_qs(p.query).get('pageid', [None])[0]
    if pageid is not None:
      return int(pageid)

    # standard page
    if p.path.endswith('.html'):
      return int(os.path.splitext(os.path.basename(p.path))[0])

    return None

  def get_absolute_uri(self, uripart):
    return urlparse.urljoin(self._base, uripart)

  ########################################
  # Wiki-wide APIs
  ########################################

  def search(self, keyword=None, is_and=True, wiki_syntax=False, complete=True):
    if keyword is not None:
      andor = 'and' if is_and else 'or'
      search_field = 'source' if wiki_syntax else ''
      cmpmode = 'cmp' if complete else ''
      return '{0}/?cmd=search&keyword={1}&andor={2}&search_field={3}&cmp={4}'.format(
          self._base, urlquote(keyword), andor, search_field, cmpmode)
    else:
      return '{0}/search'.format(self._base)

  def tags(self, sort='', index=1):
    return '{}/tag/?sort={}&p={}'.format(self._base, sort, index)

  def tag(self, tag_name='', index=1):
    if tag_name == '':
      warnings.warn(
        'Specifying empty string to tag_name to get URL for list of tags is deprecated.'
        ' Use .tags() instead.',
        DeprecationWarning)
    return '{0}/tag/{1}?p={2}'.format(self._base, urlquote(tag_name), index)

  def new(self):
    return '{0}/new'.format(self._base)

  def list(self, sort='update', index=1):
    """
    ``sort`` can be any of ``update``, ``create``, ``create_desc`` or ``pagename``.
    """
    return '{0}/list?sort={1}&pp={2}'.format(self._base, sort, index)

  def contact(self):
    return '{0}/contact'.format(self._base)

  ########################################
  # Backup APIs
  ########################################

  def _backup(self, action, page_id=None, generation=None):
    url = '{0}/?cmd=backup&action={1}'.format(self._base, action)
    if page_id is not None:
      url += '&pageid={0}'.format(page_id)
    if generation is not None:
      url += '&num={0}'.format(generation)
    return url

  def backup_list(self, page_id=None):
    return self._backup('list', page_id)

  def backup_source(self, page_id, generation):
    return self._backup('source', page_id, generation)

  def backup_show(self, page_id, generation):
    return self._backup('show', page_id, generation)

  def backup_diff(self, page_id, generation):
    return self._backup('diff', page_id, generation)

  def backup_nowdiff(self, page_id, generation):
    return self._backup('nowdiff', page_id, generation)

  ########################################
  # Page APIs
  ########################################

  def page(self, page_id):
    return '{0}/pages/{1}.html'.format(self._base, page_id)

  def diff(self, page_id):
    return '{0}/diffx/{1}.html'.format(self._base, page_id)

  def word(self, page_id, keyword):
    return '{0}/?cmd=word&pageid={1}&word={2}&type=normal'.format(self._base, page_id, urlquote(keyword))

  def edit(self, page_id, menu=None, mode=None):
    if menu is not None:
      warnings.warn(
        '`menu` option will be removed in the future release. Use `mode` option instead.',
        DeprecationWarning)
    else:
      menu = True

    if mode is None:
      mode = 'editx' if menu else 'editxx'

    if mode not in [
          'pedit',    # standard
          'editx',    # simple (with menu)
          'editxx',   # simple (without menu)
        ]:
      raise ValueError('invalid mode: {}'.format(mode))
    return '{0}/{1}/{2}.html'.format(self._base, mode, page_id)

  def rename(self, page_id):
    return '{0}/renamex/{1}.html'.format(self._base, page_id)

  def chmod(self, page_id):
    return '{0}/chmod/{1}.html'.format(self._base, page_id)

  def chkind(self, page_id):
    return '{0}/chkind/{1}.html'.format(self._base, page_id)

  def upload(self, page_id):
    return '{0}/upload/{1}.html'.format(self._base, page_id)



# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals

import math
import re
from unittest import TestCase

from atwiki.core import AtWikiAPI
from atwiki.uri import AtWikiURI

from . import TEST_BASE_URI

class AtWikiAPITest(TestCase):
  def setUp(self):
    self._api = AtWikiAPI(AtWikiURI(TEST_BASE_URI))

  def test_get_list(self):
    results = list(self._api.get_list())
    self.assertEqual(len(results), 19)

  def test_get_list_tag(self):
    results = list(self._api.get_list('tag01'))
    self.assertEqual(len(results), 1)
    self.assertEqual(results[0]['id'], 18)
    self.assertEqual(results[0]['name'], 'Test_atwiki.test.test_core:AtWikiAPITest:test_get_list_tag')

  def test_get_tags(self):
    results = list(self._api.get_tags())
    self.assertEqual(len(results), 1)
    self.assertEqual(results[0]['name'], 'tag01')
    self.assertEqual(results[0]['weight'], 1)

  def test_get_source(self):
    self.assertEqual(self._api.get_source(14, 0),
                     'テスト1\nテスト2\n\nテスト3\nテスト4\n\n\nテスト5')
    self.assertEqual(self._api.get_source(14, 1),
                     'テスト1\nテスト2\n\nテスト3\nテスト4')

  def test_get_source_special(self):
    self.assertEqual(self._api.get_source(15, 0),
                     '"テスト1"<br>&\n"テスト2"<br>&')

  def test_get_source_invalid(self):
    self.assertRaises(IndexError, self._api.get_source, 15, 100000)

  def test_search(self):
    results = list(self._api.search('SearchKeyword01 SearchKeyword02'))
    self.assertEqual(len(results), 1)
    self.assertEqual(results[0]['name'], 'Test_atwiki.test.test_core:AtWikiAPITest:test_search')
    self.assertEqual(results[0]['snippet'], 'SearchKeyword01  SearchKeyword02')

  def test_search_or(self):
    results = list(self._api.search('SearchKeyword01 SearchKeyword02', False))
    self.assertEqual(len(results), 2)
    self.assertEqual(results[0]['name'], 'Test_atwiki.test.test_core:AtWikiAPITest:test_search')
    self.assertEqual(results[0]['snippet'], 'SearchKeyword01  SearchKeyword02')
    self.assertEqual(results[1]['name'], 'Test_atwiki.test.test_core:AtWikiAPITest:test_search_or')
    self.assertEqual(results[1]['snippet'], 'SearchKeyword02')

  def test_search_none(self):
    results = list(self._api.search('no_result_expected_for_this'))
    self.assertEqual(len(results), 0)


class PagerizeTest(TestCase):
    def setUp(self):
        self._uri = AtWikiURI('https://w.atwiki.jp/hmiku')
        self._api = AtWikiAPI(self._uri)

    def test_get_list(self):
        top_page = next(self._api.get_list())
        assert top_page == {'id': 1, 'name': 'トップページ'}

        soup = self._api._request(self._uri.list(sort='create', index=1))
        text = soup.find('div', class_='pagelist').text
        m = re.search(r'計 (\d+) ページ / 1 から 100 を表示', text)
        assert m is not None
        count = int(m.group(1))
        assert 45000 < count < 90000
        last_index = math.ceil(count / 100)

        # Get list from the last page.
        # N.B. The page counter is not updated immediately.
        pages = list(self._api.get_list(_start=last_index))
        expected = (count % 100)
        assert max(0, (expected - 50)) < len(pages) < min(100, (expected + 50))

        top_page = next(self._api.get_list(_start=last_index + 1))
        assert top_page == {'id': 1, 'name': 'トップページ'}

    def test_get_list_tag(self):
        soup = self._api._request(self._uri.tag('曲', index=1))
        last_index = 1
        for link in soup.find('div', class_='cmd_tag').find_all('a'):
            if not link.attrs['href'].endswith('&p={}'.format(last_index + 1)):
                break
            last_index += 1
        pages = list(self._api.get_list('曲', _start=last_index))
        assert 1 <= len(pages) <= 50

        pages = list(self._api.get_list('曲', _start=last_index + 1))
        assert len(pages) == 0

    def test_get_tags(self):
        song = next(self._api.get_tags('num'))
        assert song['name'] == '曲'
        assert 35000 < song['weight'] < 70000

        not_song = next(self._api.get_tags('num', _start=2))
        assert not_song['name'] != '曲'



# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals

from unittest import TestCase

from atwiki.tools.dump import AtWikiDump

from .. import TEST_BASE_URI

class AtWikiDumpTest(TestCase):
  def test_simple(self):
    self.assertRaises(IOError, AtWikiDump.start, ['-o', '/tmp/non/existing/directory', TEST_BASE_URI])



# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals

from unittest import TestCase

from atwiki.parser import AtWikiStripper

INPUT = r"""
// This is a comment.

&nivocideo(url) &color(#ffffff){white} and black

[[Link1]] / [[Link2>URL]]
Styles: ''bold'' '''italic''' %%del%% ''bold'' 
Special: '' A ✔︎ ( ) { } \ / ! '' 


#right(aaa){inline} 
#comment() 
#region
+item 1
++item 2
-item 1
--item 2
-+item 3
*head 1
**head 2
>quote 1
>>quote 2
 pre
---- 
#exk(xxx){{{
block 1
}}}
#exk(){{{{{
block 2
}}}}}
"""

OUTPUT = r"""

white and black

Link1 / Link2
Styles: bold italic del bold 
Special:  A ✔︎ ( ) { } \ / !  


inline


item 1
item 2
item 1
item 2
+item 3
head 1
head 2
quote 1
quote 2
pre

block 1
block 2"""

class AtWikiStripperTest(TestCase):
  def test_doc(self):
    stripper = AtWikiStripper(INPUT)
    self.assertEqual(OUTPUT, stripper.text())

  def test_single(self):
    stripper = AtWikiStripper('[[Test]]')
    self.assertEqual('Test', stripper.text())



# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals

try:
  # Python 3
  from urllib.request import urlopen, Request
except ImportError:
  # Python 2
  from urllib2 import urlopen, Request

import re
import time
from collections import namedtuple

import bs4
import bs4.element
from bs4 import BeautifulSoup

from .uri import AtWikiURI

class AtWikiAPI(object):
  _TAG_WEIGHT_PATTERN = re.compile(r'\((\d+)\)$')  # "タグ名(1)"

  def __init__(self, uri, **kwargs):
    self._uri = uri
    self._user_agent = kwargs.get('user_agent', 'Mozilla/5.0 (AtWikiPython)')
    self._sleep = kwargs.get('sleep', 10)  # in seconds
    self._last_request = 0.0  # epoch second last request has made

  def get_list(self, tag=None, _start=1):
    index = _start
    while True:
      count = 0
      if tag:
        soup = self._request(self._uri.tag(tag, index))
        links = soup.find('div', attrs={'class': 'cmd_tag'}).find('ul').select('a')
        pager = soup.find('div', attrs={'class': 'cmd_tag'}).select_one('a[href$="?&p={}"]'.format(index + 1))
      else:
        soup = self._request(self._uri.list('create', index))
        links = soup.find('table', attrs={'class': 'pagelist'}).findAll('a', href=True, title=True)
        pager = soup.find('ul', attrs={'class': 'atwiki_pagination'})
        if pager is not None:
          pager = pager.select_one('a[href$="&pp={}"]'.format(index + 1))
      is_end = (pager is None or len(links) == 0)
      for link in links:
        page_id = self._uri.get_page_id_from_uri(link.attrs['href'])
        page_name = link.text.strip()
        if page_id:
          count += 1
          yield {'id': page_id, 'name': page_name}
      if count == 0 or is_end: break
      index += 1

  def get_tags(self, sort='', _start=1):
    index = _start
    while True:
      count = 0
      soup = self._request(self._uri.tags(sort, index))
      links = soup.find('div', attrs={'class': 'cmd_tag'}).findAll('a', attrs={'class': 'tag'})
      for link in links:
        tag_name = link.text
        tag_weight = 0
        m = self._TAG_WEIGHT_PATTERN.search(link.attrs['title'])
        if m:
          tag_weight = int(m.group(1))
        count += 1
        yield {'name': tag_name, 'weight': tag_weight}
      if count == 0: break

      # Find "次の500件" link.
      pager = soup.find('div', attrs={'class': 'cmd_tag'}).select_one('a[href$="/tag/?p={}"]'.format(index + 1))
      if not pager:
        break
      index += 1

  def get_source(self, page_id, generation=0):
    soup = self._request(self._uri.backup_source(page_id, generation))
    pre = soup.find('pre', attrs={'class': 'cmd_backup'})
    if not pre:
      raise IndexError('page {0}: generation {1} out of range'.format(page_id, generation))
    return pre.text.replace('\r', '')

  def search(self, keyword, is_and=True, wiki_syntax=False, complete=True):
    soup = self._request(self._uri.search(keyword, is_and, wiki_syntax, complete))
    lis = soup.find('div', id='wikibody').findAll('li')[:-1]  # drop last item (link to https://atwiki.jp/wiki/<keyword>)
    for li in lis:
      a = li.find('a')
      if not a: continue
      name = a.text
      snippet = None
      for sib in a.next_siblings:
        if snippet is None:
          if sib.name == 'br': snippet = ''
          continue
        if isinstance(sib, bs4.element.Tag):
          snippet += sib.text
        else:
          snippet += str(sib)
      snippet = '' if snippet is None else snippet.strip()
      yield {'name': name, 'snippet': snippet}

  def _request(self, url, data=None):
    req = Request(url, headers={'User-Agent': self._user_agent}, data=data)
    sleep = self._last_request + self._sleep - time.time()
    if 0 < sleep:
      time.sleep(sleep)
    self._last_request = time.time()
    return BeautifulSoup(urlopen(req).read(), 'html5lib')



# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals

from unittest import TestCase

from atwiki.uri import AtWikiURI

from . import TEST_BASE_URI

class AtWikiURITest(TestCase):
  def test_str(self):
    base = TEST_BASE_URI
    self.assertEqual(base, str(AtWikiURI(base)))

  def test_path_input(self):
    base = TEST_BASE_URI
    uri = AtWikiURI(base)

    self.assertEqual(uri.get_page_id_from_uri('{0}/backupx/123/list.html'.format(base)), 123)
    self.assertEqual(uri.get_page_id_from_uri('{0}/?cmd=backup&action=show&pageid=456&num=1'.format(base)), 456)
    self.assertEqual(uri.get_page_id_from_uri('{0}/pages/789.html'.format(base)), 789)

  def test_path_output(self):
    base = TEST_BASE_URI
    uri = AtWikiURI(base)

    self.assertEqual(uri.search(), '{0}/search'.format(base))
    self.assertEqual(uri.search('test'), '{0}/?cmd=search&keyword=test&andor=and&search_field=&cmp=cmp'.format(base))
    self.assertEqual(uri.search('test', is_and=False), '{0}/?cmd=search&keyword=test&andor=or&search_field=&cmp=cmp'.format(base))
    self.assertEqual(uri.search('test', wiki_syntax=True), '{0}/?cmd=search&keyword=test&andor=and&search_field=source&cmp=cmp'.format(base))
    self.assertEqual(uri.search('test', complete=False), '{0}/?cmd=search&keyword=test&andor=and&search_field=&cmp='.format(base))
    self.assertEqual(uri.tags(), '{0}/tag/?sort=&p=1'.format(base))
    self.assertEqual(uri.tags('num', 2), '{0}/tag/?sort=num&p=2'.format(base))
    self.assertEqual(uri.tag(), '{0}/tag/?p=1'.format(base))  # deprecated
    self.assertEqual(uri.tag('test'), '{0}/tag/test?p=1'.format(base))
    self.assertEqual(uri.tag('test', 2), '{0}/tag/test?p=2'.format(base))
    self.assertEqual(uri.new(), '{0}/new'.format(base))
    self.assertEqual(uri.list(), '{0}/list?sort=update&pp=1'.format(base))
    self.assertEqual(uri.list('create'), '{0}/list?sort=create&pp=1'.format(base))
    self.assertEqual(uri.list('create', 2), '{0}/list?sort=create&pp=2'.format(base))
    self.assertEqual(uri.contact(), '{0}/contact'.format(base))

    self.assertEqual(uri.backup_list(), '{0}/?cmd=backup&action=list'.format(base))
    self.assertEqual(uri.backup_list(10), '{0}/?cmd=backup&action=list&pageid=10'.format(base))
    self.assertEqual(uri.backup_source(10, 1), '{0}/?cmd=backup&action=source&pageid=10&num=1'.format(base))
    self.assertEqual(uri.backup_show(10, 1), '{0}/?cmd=backup&action=show&pageid=10&num=1'.format(base))
    self.assertEqual(uri.backup_diff(10, 1), '{0}/?cmd=backup&action=diff&pageid=10&num=1'.format(base))
    self.assertEqual(uri.backup_nowdiff(10, 1), '{0}/?cmd=backup&action=nowdiff&pageid=10&num=1'.format(base))

    self.assertEqual(uri.page(10), '{0}/pages/10.html'.format(base))
    self.assertEqual(uri.diff(10), '{0}/diffx/10.html'.format(base))
    self.assertEqual(uri.word(10, 'test'), '{0}/?cmd=word&pageid=10&word=test&type=normal'.format(base))
    self.assertEqual(uri.edit(10), '{0}/editx/10.html'.format(base))
    self.assertEqual(uri.edit(10, False), '{0}/editxx/10.html'.format(base))
    self.assertEqual(uri.rename(10), '{0}/renamex/10.html'.format(base))
    self.assertEqual(uri.chmod(10), '{0}/chmod/10.html'.format(base))
    self.assertEqual(uri.chkind(10), '{0}/chkind/10.html'.format(base))
    self.assertEqual(uri.upload(10), '{0}/upload/10.html'.format(base))



# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals

import sys
import time
import optparse
import json

from ..core import AtWikiAPI
from ..uri import AtWikiURI

class AtWikiDump(object):
  """
  ``atwiki-dump`` is a simple tool to dump every Wiki source.

  Restrictions:
    - Contents other than Wiki source, page number and page name will not be dumped.
      This means that followings are not included in the dump:
        - WYSIWYG mode (i.e., syntax of the source)
        - history backup
        - tags
        - ACLs
    - No consistency between pages.
  """

  @classmethod
  def start(cls, args):
    USAGE = 'atwiki-dump [--output-dir OUTPUT_DIR] WIKI_URI'

    parser = optparse.OptionParser(description='@wiki Dump Tool', usage=USAGE)
    parser.add_option('--output-dir', '-o', type='string', default='.',
                      help='directory to output Wiki source')

    (parsed, target) = parser.parse_args(args)
    if len(target) != 1:
      parser.error('A single WIKI_URI must be specified')

    output_dir = parsed.output_dir
    target_site = target[0]

    api = AtWikiAPI(AtWikiURI(target_site))
    page_meta = {'version': 1, 'meta': {}}

    for page in api.get_list():
      page_id = page['id']
      page_name = page['name']
      page_meta['meta'][page_id] = {'name': page_name}
      path = '{0}/{1}.wiki'.format(output_dir, page_id)

      print('dumping: page {0} ({1}) to {2}'.format(page_id, page_name, path))
      page_src = api.get_source(page_id)
      with open(path, 'w') as f:
        f.write(page_src)
      time.sleep(10)

    path = '{0}/meta.json'.format(output_dir)
    print('dumping: meta data to {0}'.format(path))
    with open(path, 'w') as f:
      json.dump(page_meta, f)
    print('done!')

    return 0

def main():
  sys.exit(AtWikiDump.start(sys.argv[1:]))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

def _read(filename):
  with open(filename) as f:
    return f.read()

# Load package version.
exec(_read('atwiki/_version.py'))

setup(name='atwiki-python',
      version='.'.join(str(x) for x in VERSION),
      description='Atwiki Client Library',
      long_description=_read('README.rst'),
      url='https://github.com/kmaehashi/atwiki-python',
      author='Kenichi Maehashi',
      author_email='webmaster@kenichimaehashi.com',
      license='MIT',
      classifiers=[
          'Intended Audience :: Developers',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python',
          'Programming Language :: Python :: 2',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.7',
          'Programming Language :: Python :: 3.8',
          'Programming Language :: Python :: 3.9',
          'Programming Language :: Python :: 3.10',
      ],
      packages=find_packages(),
      test_suite = 'atwiki.test',
      entry_points={
          'console_scripts': ['atwiki-dump=atwiki.tools.dump:main'],
      },
      python_requires='>=2.7,!=3.0.*,!=3.1.*,!=3.2.*,!=3.3.*,!=3.4.*,!=3.5.*,!=3.6.*',
      install_requires=[
          'BeautifulSoup4',
          'html5lib',
      ],
)
